#include "SREnemyCommand.h"
#include "SRGameObject.h"
#include "WindowDefs.h"

//#include <iostream>

SREnemyCommand::SREnemyCommand( const char formation[] )
{
    char type[5];
    *((int*)type) = Type();
    type[4] = 0;
//    cout << "Object of type " << type << " allocated with address " << this << endl;
    fNumShips = formation[0];
    fNumLivingShips = fNumShips;
    fShips = new ShipPosition[fNumShips];
    
    char i;
    int location = 1;
    char maxX = 0, minX = 0, maxY = 0, minY = 0;
    
    // For every entry in the formation structure (see .h for info),
    // create a ship position, and a new ship.
    for( i=0; i < fNumShips; i++ )
    {
        bool isIndy;

        switch(formation[location+3])
        {
            default:
                fShips[i].ship = new SREnemySprite(140, this);
                isIndy = false;
                break;
        }

        fShips[i].id = i;
        fShips[i].xOffset = formation[location];
        fShips[i].yOffset = formation[location + 1];
        fShips[i].blockedBy = formation[location + 2];
        fShips[i].isIndy = isIndy;
        
        if( fShips[i].xOffset < minX )
            minX = fShips[i].xOffset;
        else if( fShips[i].xOffset > maxX )
            maxX = fShips[i].xOffset;
        
        
        if(fShips[i].blockedBy < 0)
        {
            fShips[i].ship->AllowFiring();
        }
        
        if( fShips[i].yOffset < minY )
        {
            minY = fShips[i].yOffset;
        }
        else if( fShips[i].yOffset > maxY )
        {
            maxY = fShips[i].yOffset;
        }
            
        location += 4;
    }
    
    fWidth = maxX - minX;
    fHeight = maxY - minY;
    
    ResetPosition();
}

SREnemyCommand::~SREnemyCommand()
{
    char i;
    // delete all the leftover ships
    for( i=0; i<fNumShips; i++ )
    {
        if(fShips[i].ship != 0)
            SRGameObject::instance->Recycle(fShips[i].ship);
    }
    
    delete [] fShips;
}

void SREnemyCommand::ResetPosition()
{
    // place the fleet in the top-left of the screen.
    fX = fWidth/2 + (4*kCurbWidth);
    fY = fHeight/2 + (4*kCurbWidth);
    fVelocityX = kRightVelocity;
}

void SREnemyCommand::Think()
{
    // if there are no ships left, do something
    if(fNumLivingShips == 0)
    {
        fNumLivingShips = -1;
        SRGameObject::instance->LevelUp();
        return;
    }
    else if(fNumLivingShips < 0)
    {
        return;
    }

    // Are we at an edge?  If so, appropriately move us.
    // If not, let's just keep going as is...
    if( fX < (fWidth/2) )
    {
        fVelocityX = kRightVelocity;
        fAccelY = -1;
    }
    else if( fX < (fWidth/2 + kCurbWidth) )
    {
        fAccelY = 1;
    }
    else if( (kGameWindowWidth - fX) < (fWidth/2) )
    {
        fVelocityX = kLeftVelocity;
        fAccelY = -1;
    }
    else if( (kGameWindowWidth - fX) < (fWidth/2 + kCurbWidth) )
    {
        fAccelY = 1;
    }
    else
    {
        fAccelY = 0;
        fVelocityY = 0;
    }
    
    // If we pass the bottom, it's game over for the player...
    if( fY > (kGameWindowHeight - kBottomBoundary) )
    {
        SRGameObject::instance->GameOver();
    }
    
    // Place each ship in the fleet, where it should be.
    int i;
    for( i=0; i<fNumShips; i++ )
    {
        if(fShips[i].ship != 0  && !fShips[i].isIndy)
        {
            fShips[i].ship->setX( fX + fShips[i].xOffset );
            fShips[i].ship->setY( fY + fShips[i].yOffset );
        }
    }
}

bool SREnemyCommand::Hit( SRObject */*other*/ )
{
// no action to hit...
    return false;
}

// Called whenever a ship in the fleet bites it...
void SREnemyCommand::ChildDied( SRObject *ship )
{
    if(ship == 0) return;

    char i;
    char id;
    char dependID;
    bool couldFire;
    
    // Find the ship's id, and the id of the ship that blocks it
    for( i=0; i<fNumShips; i++ )
    {
        if( fShips[i].ship == ship )
        {
            id = fShips[i].id;
            dependID = fShips[i].blockedBy;
            couldFire = fShips[i].ship->FiringAllowed();
            break;
        }
    }
    
    // now search for ships that can now fire...
    for( i=0; i<fNumShips; i++ )
    {
        if(fShips[i].blockedBy == id)
        {
            if( couldFire )
            {
                if(fShips[i].ship != 0) fShips[i].ship->AllowFiring();
            }
            else
            {
                // if the ship that just bought it couldn't fire,
                // we should let its dependants know who it was waiting on to fire
                fShips[i].blockedBy = dependID;
            }
        }
    }

    // we just lost a ship... we must mourn it...
    fNumLivingShips--;
    fShips[id].ship = 0;
}
