#ifndef __SR_GAME_OBJECT_H__
#define __SR_GAME_OBJECT_H__

#include "SRObjectList.h"

/************************************************************************
 * Game levels format:
 * Field                        Size
 * ---------------------------------------------------------
 *  Number of levels (n)        1 byte
 *  Level intro screen base     1 unsigned byte
 *  n Level records             variable
 *
 * Level record format:
 * Field                        Size
 * ---------------------------------------------------------
 *  Reserved 1                  1 byte
 *  Reserved 2                  1 byte
 *  Bonuses Allowed             2 bytes (bit mask of bonuses)
 *  Standard Options            2 bytes   "   "         "
 *  Intro screen offset         1 unsigned byte
 *  Formation record size       2 bytes (size in bytes)
 *  Formation record            variable (equal to previous field's value)
 *
 * note: multi-byte values are stored big-endian
 ************************************************************************/

#define kCatchAlert         130

class SREnemyCommand;
class SRIconSprite;

class SRGameObject
{
public:
    SRGameObject();
    ~SRGameObject();
    
    void NewGame();
    void LevelUp();
    void GameOver();
    
    void Draw();
    void Think();
    void Move();
    SRObjectList *TestForHit( SRObject *querant );
    void Recycle(SRObject *object);
    
    static bool Paused() { return fPause; }
    static void TogglePause();
    
    int Score() { return fScore; }
    void Scored( int points ) { fScore += points; }
    
    int TimesDied() { return fTimesDied; }
    void PlayerDied() { fTimesDied++; }

public:
    static SRGameObject *instance;
    SRObjectList *gameObjectList() { return fList; }

protected:
    void CreateStarField();

protected:
    static bool          fPause;
    SRObjectList        *fList;
    SRObjectList        *fStarField;
    SREnemyCommand      *fEnemyCommand;
    
    // Game stats
    int                  fScore;
    int                  fTimesDied;
};

#endif // __SR_GAME_OBJECT_H__