#include "SRPlayerSprite.h"
#include "SRAmmoSprite.h"
#include "SRGameObject.h"
#include "WindowDefs.h"
#include "SRSoundManager.h"

//#include <iostream>

SRPlayerSprite *SRPlayerSprite::fInstance = 0;

namespace
{
    int start;
}

SRPlayerSprite::SRPlayerSprite( int startingID ) : SRIconSprite(0),
    fDirection( DirectionHalt ), fAmmoTimer(0), fFireNow(false)
{
    start = startingID;
    char type[5];
    *((int*)type) = Type();
    type[4] = 0;
//    cout << "Object of type " << type << " allocated with address " << this << endl;
    fInstance = this;
    
    fHeight = 30;
    fWidth = 23;
    
    ResetPosition();
    
    // Load the ship's icons
//    fAmmoIcon = GetCIcon( startingID + kAmmoIcon );
//    if( fAmmoIcon == 0 ) cout << "Couldn't load ammo icon..." << endl;
    
    fLeftIcon = GetCIcon( startingID + kLeftIcon );
//    if( fLeftIcon == 0 ) cout << "Couldn't load left icon..." << endl;
    
    fRightIcon = GetCIcon( startingID + kRightIcon );
//    if( fRightIcon == 0 ) cout << "Couldn't load right icon..." << endl;
    
    fStationaryIcon = GetCIcon( startingID + kStationaryIcon );
}

SRPlayerSprite::~SRPlayerSprite()
{
    // get rid of the ship's icons
    //DisposeCIcon(fAmmoIcon);
    DisposeCIcon(fLeftIcon);
    DisposeCIcon(fRightIcon);
    DisposeCIcon(fStationaryIcon);
    
    fInstance = 0;
}

void SRPlayerSprite::ResetPosition()
{
    // put the ship in the middle of the bottom of the screen...
    fX = kGameWindowWidth / 2;
    fY = kGameWindowHeight - kBoundary;
    fVelocityX = 0;
    fVelocityY = 0;
    fDirection = DirectionHalt;
}

void SRPlayerSprite::TakeCommand( Command cmd )
{
    // if the command is a fire, we fire, otherwise it's a move command
    switch(cmd)
    {
        case WeaponFire:
            fFireNow = true;
            break;
            
        default:
            fDirection = cmd;
            break;
    }
}

void SRPlayerSprite::Think()
{
    // If we can fire, and we want to fire, then do it!
    if(fAmmoTimer > 0) fAmmoTimer--;
    if( fFireNow && (fAmmoTimer == 0) )
    {
        SRAmmoSprite *ammo = new SRAmmoSprite(GetCIcon( start + kAmmoIcon ), this);
        
        ammo->setX( fX );
        ammo->setY( fY + kAmmoDistance );
        
        ammo->pushY( kAmmoForce );
        
        fFireNow = false;
        fAmmoTimer = kAmmoCountdown;
        
        SRSoundManager::PlaySoundEffect( SRSoundManager::SFXplayerFires );
    }
    
    // Have we passed a boundary?  If so, let's stop!
    if( fX <= kBoundary )
    {
        fDirection = DirectionHalt;
        fX = kBoundary + 1;
    }
    else if( fX >= (kGameWindowWidth - kBoundary) )
    {
        fDirection = DirectionHalt;
        fX = (kGameWindowWidth - (kBoundary + 1));
    }
    
    if( fY >= (kGameWindowHeight - kBoundary) )
    {
        fY = (kGameWindowHeight - kBoundary) - 1;
        fVelocityY = 0;
    }
    else if( fY <= (kGameWindowHeight - kBoundary - kZone) )
    {
        fY = (kGameWindowHeight - kBoundary - kZone) + 1;
    }
    
    // Depending on the direction, we choose a speed
    switch(fDirection)
    {
        case DirectionHalt:
            fVelocityX = 0;
            fVelocityY = 0;
            break;
            
        case DirectionLeft:
            fVelocityX = -2;
            break;
            
        case DirectionRight:
            fVelocityX = 2;
            break;
        
        case DirectionUp:
            fVelocityY = -2;
            break;
        
        case DirectionDown:
            fVelocityY = 2;
            break;
    }
    
    // give feedback of the 
    if(fVelocityX < 0)
        fCurrentIcon = fLeftIcon;
    else if(fVelocityX > 0)
        fCurrentIcon = fRightIcon;
    else
        fCurrentIcon = fStationaryIcon;
}

// one of the bullets hit something solid!
void SRPlayerSprite::ChildDied( SRObject * )
{
}

// something hit us!
bool SRPlayerSprite::Hit( SRObject *other )
{
    if( other->Type() == kEnemyAmmoType || other->Type() == kGenericAmmoType )
    {
        SRSoundManager::PlaySoundEffect( SRSoundManager::SFXplayerHit );
        SRGameObject::instance->PlayerDied();
        ResetPosition();
        return true;
    }
    
    return false;
}